@@ -5,9 +5,10 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-#pylint: disable=bare-except
+# pylint: disable=bare-except
 
 import datetime
+import shutil
 from subprocess import Popen, PIPE, STDOUT
 import socket
 import platform
@@ -16,53 +17,67 @@ import re
 import sys
 
 
-def bytestostr(x):
+SRC_INFO_FILE = "./xpra/src_info.py"
+BUILD_INFO_FILE = "./xpra/build_info.py"
+
+
+def bytestostr(x) -> str:
     if isinstance(x, bytes):
         return x.decode("latin1")
     return str(x)
 
 
-def update_properties(props, filename):
+def update_properties(props: dict, filename: str) -> None:
     eprops = get_properties(filename)
     for key,value in props.items():
         set_prop(eprops, key, value)
     save_properties(eprops, filename)
 
-def save_properties(props, filename):
+
+def write_props(f, props: dict, prefix="") -> None:
+    def u(v) -> bytes:
+        if isinstance(v, bytes):
+            return v
+        try:
+            v = v.decode()
+        except (UnicodeDecodeError, AttributeError):
+            v = str(v)
+        try:
+            return v.encode("utf-8")
+        except UnicodeEncodeError:
+            return v.encode("latin1")
+
+    def w(v) -> None:
+        f.write(u(v))
+
+    def fmt(value) -> str:
+        if isinstance(value, (bool, tuple, int)):
+            return str(value)
+        s = bytestostr(value).replace("'", "\\'")
+        return f"'{s}'"
+
+    for name in sorted(props.keys()):
+        if prefix:
+            w(f"{prefix}{fmt(name)}")
+        else:
+            w(name)
+        w(f"={fmt(props[name])}\n")
+
+
+def save_properties(props: dict, filename: str) -> None:
     if os.path.exists(filename):
         try:
             os.unlink(filename)
         except OSError:
-            print("WARNING: failed to delete %s" % filename)
-    def u(v):
-        try:
-            v = v.decode()
-        except Exception:
-            v = str(v)
-        try:
-            return v.encode("utf-8")
-        except UnicodeDecodeError:
-            return v
+            print(f"WARNING: failed to delete {filename!r}")
     with open(filename, mode='wb') as f:
-        def w(v):
-            f.write(u(v))
-        for name in sorted(props.keys()):
-            value = props[name]
-            s = bytestostr(value).replace("'", "\\'")
-            w(name)
-            w("=")
-            quote_it = not isinstance(value, (bool, tuple, int))
-            if quote_it:
-                w("'")
-            w(s)
-            if quote_it:
-                w("'")
-            w("\n")
-    print("updated %s with:" % filename)
+        write_props(f, props)
+    print(f"updated {filename!r} with:")
     for k in sorted(props.keys()):
         print("* %s = %s" % (str(k).ljust(20), bytestostr(props[k])))
 
-def get_properties(filename):
+
+def get_properties(filename: str) -> dict:
     props = dict()
     if not os.path.exists(filename):
         return props
@@ -71,7 +86,7 @@ def get_properties(filename):
             try:
                 s = line.decode("utf-8")
             except UnicodeDecodeError:
-                #str cannot be decoded!
+                # str cannot be decoded!
                 s = str(line)
             s = s.strip()
             if not s:
@@ -80,24 +95,25 @@ def get_properties(filename):
                 continue
             parts = s.split("=", 1)
             if len(parts)<2:
-                print("missing equal sign: %s" % s)
+                print(f"missing equal sign in {s!r}")
                 continue
             name = parts[0]
             value = parts[1]
             if not value:
                 continue
-            if value[0]!="'" or value[-1]!="'":
+            if value[0] != "'" or value[-1] != "'":
                 continue
-            props[name]= value[1:-1]
+            props[name] = value[1:-1]
     return props
 
 
-def get_machineinfo():
+def get_machineinfo() -> str:
     if platform.uname()[4]:
         return platform.uname()[4]
     return "unknown"
 
-def get_cpuinfo():
+
+def get_cpuinfo() -> str:
     if platform.uname()[5]:
         return platform.uname()[5]
     if os.path.exists("/proc/cpuinfo"):
@@ -107,7 +123,8 @@ def get_cpuinfo():
                     return line.split(": ")[1].replace("\n", "").replace("\r", "")
     return "unknown"
 
-def get_status_output(*args, **kwargs):
+
+def get_status_output(*args, **kwargs) -> tuple[int, str | bytes, str | bytes]:
     kwargs["stdout"] = PIPE
     kwargs["stderr"] = STDOUT
     try:
@@ -118,10 +135,11 @@ def get_status_output(*args, **kwargs):
     stdout, stderr = p.communicate()
     return p.returncode, stdout, stderr
 
+
 def get_output_lines(cmd, valid_exit_code=0):
     try:
         returncode, stdout, stderr = get_status_output(cmd, shell=True)
-        if returncode!=valid_exit_code:
+        if returncode != valid_exit_code:
             print("'%s' failed with return code %s" % (cmd, returncode))
             print("stderr: %s" % stderr)
             return ()
@@ -132,30 +150,34 @@ def get_output_lines(cmd, valid_exit_code=0):
         return out.splitlines()
     except Exception as e:
         print("error running '%s': %s" % (cmd, e))
-    return  ()
+    return ()
 
-def get_first_line_output(cmd, valid_exit_code=0):
+
+def get_first_line_output(cmd, valid_exit_code=0) -> str:
     lines = get_output_lines(cmd, valid_exit_code)
     if lines:
         return lines[0]
-    return  ""
+    return ""
 
-def get_nvcc_version():
-    for p in ("/usr/local/cuda/bin", "/opt/cuda/bin", ""):
-        nvcc = os.path.join(p, "nvcc")
-        if p=="" or os.path.exists(nvcc):
-            cmd = "%s --version" % (nvcc)
+
+def get_nvcc_version() -> str:
+    if sys.platform == "darwin":
+        return ""
+    for nvcc in ("/usr/local/cuda/bin/nvcc", "/opt/cuda/bin/nvcc", shutil.which("nvcc")):
+        if nvcc and os.path.exists(nvcc):
+            cmd = f"{nvcc} --version"
             lines = get_output_lines(cmd)
             if lines:
                 vline = lines[-1]
                 vpos = vline.rfind(", V")
                 if vpos>0:
                     return vline[vpos+3:]
-    return None
+    return ""
+
 
 def get_compiler_version():
     cc_version = "%s --version" % os.environ.get("CC", "gcc")
-    if sys.platform=="darwin":
+    if sys.platform == "darwin":
         lines = get_output_lines(cc_version)
         for line in lines:
             if line.startswith("Apple"):
@@ -163,8 +185,9 @@ def get_compiler_version():
         return None
     return get_first_line_output(cc_version)
 
+
 def get_linker_version():
-    if sys.platform=="darwin":
+    if sys.platform == "darwin":
         ld_version = "%s -v" % os.environ.get("LD", "ld")
         lines = get_output_lines(ld_version)
         for line in lines:
@@ -175,24 +198,25 @@ def get_linker_version():
     return get_first_line_output(ld_version)
 
 
-def set_prop(props, key, value):
+def set_prop(props, key: str, value) -> None:
     if value is None:
         return
-    if value!="unknown" or props.get(key) is None:
+    if value not in ("unknown", "") or props.get(key) is None:
         props[key] = value
 
-def get_platform_name():
-    #better version info than standard python platform:
+
+def get_platform_name() -> str:
+    # better version info than standard python platform:
     if sys.platform.startswith("sun"):
-        #couldn't find a better way to distinguish opensolaris from solaris...
+        # couldn't find a better way to distinguish opensolaris from solaris...
         with open("/etc/release", "r", encoding="latin1") as f:
             data = f.read()
         if data and str(data).lower().find("opensolaris"):
             return "OpenSolaris"
         return "Solaris"
-    if sys.platform.find("darwin")>=0:
+    if sys.platform == "darwin":
         try:
-            #ie: MacOS 10.14.6
+            # ie: MacOS 10.14.6
             return "MacOS %s" % platform.mac_ver()[0]
         except (AttributeError, TypeError, IndexError):
             return "MacOS"
@@ -209,31 +233,35 @@ def get_platform_name():
         except OSError:
             pass
         return "Microsoft Windows"
-    if sys.platform.find("bsd")>=0:
+    if sys.platform.find("bsd") >= 0:
         return "BSD"
     try:
-        from xpra.os_util import get_linux_distribution
+        from xpra.util.system import get_linux_distribution
         ld = get_linux_distribution()
-        if ld:
-            return "Linux %s" % (" ".join(ld))
+        ldvalid = tuple(x for x in ld if x not in ("", "unknown", "n/a"))
+        if ldvalid:
+            return "Linux %s" % (" ".join(ldvalid))
     except Exception:
         pass
     return sys.platform
 
 
-BUILD_INFO_FILE = "./xpra/build_info.py"
-def record_build_info():
+def alnum(v: str | bytes) -> str:
+    return "".join(v for v in filter(str.isalnum, bytestostr(v)))
+
+
+def record_build_info() -> None:
     props = get_properties(BUILD_INFO_FILE)
     source_epoch = os.environ.get("SOURCE_DATE_EPOCH")
     if source_epoch:
-        #reproducible builds:
-        build_time = datetime.datetime.utcfromtimestamp(int(source_epoch))
+        # reproducible builds:
+        build_time = datetime.datetime.fromtimestamp(int(source_epoch), tz=datetime.timezone.utc)
         build_date = build_time.date()
     else:
-        #win32, macos and older build environments:
+        # win32, macos and older build environments:
         build_time = datetime.datetime.now()
         build_date = datetime.date.today()
-        #also record username and hostname:
+        # also record username and hostname:
         try:
             import getpass
             set_prop(props, "BUILT_BY", getpass.getuser())
@@ -246,6 +274,7 @@ def record_build_info():
     set_prop(props, "BUILD_CPU", get_cpuinfo())
     set_prop(props, "BUILD_BIT", platform.architecture()[0])
     set_prop(props, "BUILD_OS", get_platform_name())
+    set_prop(props, "BUILD_TYPE", os.environ.get("BUILD_TYPE", ""))
     try:
         from Cython import __version__ as cython_version
     except ImportError:
@@ -255,46 +284,80 @@ def record_build_info():
     set_prop(props, "COMPILER_VERSION", get_compiler_version())
     set_prop(props, "NVCC_VERSION", get_nvcc_version())
     set_prop(props, "LINKER_VERSION", get_linker_version())
-    #record pkg-config versions:
+    # record pkg-config versions:
     PKG_CONFIG = os.environ.get("PKG_CONFIG", "pkg-config")
-    for pkg in ("libc",
-                "vpx", "x264", "webp", "yuv", "nvenc", "nvfbc",
-                "avcodec", "avutil", "swscale",
-                "nvenc",
-                "x11", "xrandr", "xtst", "xfixes", "xkbfile", "xcomposite", "xdamage", "xext",
-                "gobject-introspection-1.0",
-                "gtk+-3.0", "py3cairo", "pygobject-3.0", "gtk+-x11-3.0",
-                "python3",
-                ):
-        #fugly magic for turning the package atom into a legal variable name:
-        pkg_name = pkg.lstrip("lib").replace("+", "").replace("-", "_")
-        if pkg_name.rsplit("_", 1)[-1].rstrip("0123456789.")=="":
-            pkg_name = "_".join(pkg_name.split("_")[:-1])
-        cmd = [PKG_CONFIG, "--modversion", pkg]
-        returncode, out, _ = get_status_output(cmd)
-        if returncode==0:
-            set_prop(props, "lib_"+pkg_name, out.decode().replace("\n", "").replace("\r", ""))
+    if os.name == "nt":
+        returncode, out, _ = get_status_output(["pacman", "-Q"])
+        if returncode == 0:
+            for line in out.decode().splitlines():
+                parts = line.split(" ")
+                if len(parts) != 2:
+                    continue
+                pkg_name, version = parts
+                pkg_name = alnum(pkg_name)
+                set_prop(props, f"lib_{pkg_name}", version)
+    elif sys.platform == "darwin":
+        returncode, out, _ = get_status_output(["jhbuild", "list", "-a", "-r"])
+        if returncode == 0:
+            for line in out.decode().splitlines():
+                parts = line.split(" ")
+                if len(parts) != 2:
+                    continue
+                pkg_name, version = parts
+                if pkg_name == "Modules":
+                    continue
+                pkg_name = alnum(pkg_name)
+                set_prop(props, f"lib_{pkg_name}", version.lstrip("(").rstrip(")"))
+    else:
+        for pkg in (
+            "libc",
+            "vpx", "x264", "webp", "yuv", "nvenc", "nvfbc",
+            "nvenc",
+            "x11", "xrandr", "xtst", "xfixes", "xkbfile", "xcomposite", "xdamage", "xext",
+            "gobject-introspection-1.0",
+            "gtk+-3.0", "py3cairo", "pygobject-3.0", "gtk+-x11-3.0",
+            "python3",
+        ):
+            # fugly magic for turning the package atom into a legal variable name:
+            pkg_name = alnum(pkg.lstrip("lib"))
+            if pkg_name.rsplit("_", 1)[-1].rstrip("0123456789.")=="":
+                pkg_name = "_".join(pkg_name.split("_")[:-1])
+            cmd = [PKG_CONFIG, "--modversion", pkg]
+            returncode, out, _ = get_status_output(cmd)
+            if returncode == 0:
+                set_prop(props, "lib_"+pkg_name, out.decode().replace("\n", "").replace("\r", ""))
+    # we could potentially limit this data collection to win32 and macos?
+    returncode, out, _ = get_status_output(["pip3", "freeze"])
+    if returncode == 0:
+        for line in out.decode().splitlines():
+            parts = line.split("==")
+            if len(parts) != 2:
+                continue
+            pkg_name, version = parts
+            pkg_name = alnum(pkg_name)
+            set_prop(props, f"lib_python_{pkg_name}", version)
+
     save_properties(props, BUILD_INFO_FILE)
 
 
 def get_vcs_props():
     props = {
-        "REVISION" : "unknown",
-        "LOCAL_MODIFICATIONS" : "unknown",
-        "BRANCH" : "unknown",
-        "COMMIT" : "unknown"
-        }
+        "REVISION": "unknown",
+        "LOCAL_MODIFICATIONS": 0,
+        "BRANCH": "unknown",
+        "COMMIT": "unknown"
+    }
     branch = None
     for cmd in (
         r"git branch --show-current",
-        #when in detached state, the one above does not work, but this one does:
+        # when in detached state, the one above does not work, but this one does:
         r"git branch --remote --verbose --no-abbrev --contains | sed -rne 's/^[^\/]*\/([^\ ]+).*$/\1/p'",
-        #if all else fails:
+        # if all else fails:
         r"git branch | grep '* '",
     ):
         proc = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
         out, _ = proc.communicate()
-        if proc.returncode==0:
+        if proc.returncode == 0:
             branch_out = out.decode("utf-8").splitlines()
             if branch_out:
                 branch = branch_out[0]
@@ -306,28 +369,28 @@ def get_vcs_props():
     else:
         props["BRANCH"] = branch
 
-    #use the number of changes since the last tag:
+    # use the number of changes since the last tag:
     proc = Popen("git describe --long --always --tags", stdout=PIPE, stderr=PIPE, shell=True)
     out, _ = proc.communicate()
-    if proc.returncode!=0:
+    if proc.returncode != 0:
         print("'git describe --long --always --tags' failed with return code %s" % proc.returncode)
-        return  props
+        return props
     if not out:
         print("could not get version information")
-        return  props
+        return props
     out = out.decode('utf-8').splitlines()[0]
-    #ie: out=v4.0.6-58-g6e6614571
+    # ie: out=v4.0.6-58-g6e6614571
     parts = out.split("-")
-    if len(parts)==1:
+    if len(parts) == 1:
         commit = parts[0]
         print("could not get revision number - no tags?")
         rev_str = "0"
-    elif len(parts)==3:
+    elif len(parts) == 3:
         rev_str = parts[1]
         commit = parts[2]
     else:
         print("could not parse version information from string: %s" % out)
-        return  props
+        return props
     props["COMMIT"] = commit
     try:
         rev = int(rev_str)
@@ -335,24 +398,23 @@ def get_vcs_props():
         print("could not parse revision counter from string: %s (original version string: %s)" % (rev_str, out))
         return props
 
-    if branch=="master":
-        #fake a sequential revision number that continues where svn left off,
-        #by counting the commits and adding a magic value (5014)
-        proc = Popen("git rev-list --count HEAD --first-parent",
-                                stdout=PIPE, stderr=PIPE, shell=True)
+    if branch == "master":
+        # fake a sequential revision number that continues where svn left off,
+        # by counting the commits and adding a magic value (5014)
+        proc = Popen("git rev-list --count HEAD --first-parent", stdout=PIPE, stderr=PIPE, shell=True)
         out, _ = proc.communicate()
-        if proc.returncode!=0:
+        if proc.returncode != 0:
             print("failed to get commit count using 'git rev-list --count HEAD'")
             sys.exit(1)
         rev = int(out.decode("utf-8").splitlines()[0]) + 5014
     props["REVISION"] = rev
-    #find number of local files modified:
+    # find number of local files modified:
     changes = 0
     proc = Popen("git status", stdout=PIPE, stderr=PIPE, shell=True)
     (out, _) = proc.communicate()
-    if proc.poll()!=0:
+    if proc.poll() != 0:
         print("could not get status of local files")
-        return  props
+        return props
 
     lines = out.decode('utf-8').splitlines()
     for line in lines:
@@ -362,14 +424,16 @@ def get_vcs_props():
     props["LOCAL_MODIFICATIONS"] = changes
     return props
 
-SRC_INFO_FILE = "./xpra/src_info.py"
-def record_src_info():
+
+def record_src_info() -> None:
     update_properties(get_vcs_props(), SRC_INFO_FILE)
 
-def has_src_info():
+
+def has_src_info() -> bool:
     return os.path.exists(SRC_INFO_FILE) and os.path.isfile(SRC_INFO_FILE)
 
-def has_build_info():
+
+def has_build_info() -> bool:
     return os.path.exists(BUILD_INFO_FILE) and os.path.isfile(BUILD_INFO_FILE)
 
 
@@ -390,8 +454,9 @@ def main(args):
             props.get("REVISION"),
             "M" if mods>0 else "",
             " (%s)" % commit if commit else "",
-            )
-            )
+        )
+        )
+
 
 if __name__ == "__main__":
     main(sys.argv)
