@@ -6,8 +6,8 @@
 
 import gi
 gi.require_version('Gtk', '3.0')  # @UndefinedVariable
-from gi.repository import Gtk, GLib    #pylint: disable=wrong-import-position @UnresolvedImport
-from xpra.gtk_common.gtk_util import get_default_root_window
+from gi.repository import Gtk, GLib     # pylint: disable=wrong-import-position @UnresolvedImport
+from xpra.gtk.util import get_default_root_window
 from tests.xpra.clients.fake_client import FakeClient
 
 
