@@ -1,14 +1,14 @@
 #!/usr/bin/env python3
 
-from xpra.x11.gtk3.gdk_display_source import init_gdk_display_source
+from xpra.x11.gtk3.display_source import init_gdk_display_source
 init_gdk_display_source()
 
 import gi
 gi.require_version('Gtk', '3.0')  # @UndefinedVariable
-from gi.repository import Gtk   #pylint: disable=wrong-import-position @UnresolvedImport
+from gi.repository import Gtk    # pylint: disable=wrong-import-position @UnresolvedImport
 
 from xpra.x11.gtk_x11.prop import prop_set
-from xpra.gtk_common.error import xsync
+from xpra.gtk.error import xsync
 
 def main():
     win = Gtk.Window()
