@@ -9,7 +9,7 @@ gdk_display_source.init_gdk_display_source()  # @UndefinedVariable
 from xpra.x11.bindings.window import X11WindowBindings #@UnresolvedImport
 X11Window = X11WindowBindings()
 
-from xpra.server.server_uuid import get_mode, get_uuid
+from xpra.x11.server.server_uuid import get_mode, get_uuid
 
 print(f"mode={get_mode()}")
 print(f"uuid={get_uuid()}")
